package lib;

import lib.Vec.Vec2;
import lib.Vec.Vec3;

/**
	Represents a Point in a 3D `Int` System.
	Basicly a wrapper for Vec3<Int> with some extra functions.
	`Y` represents the height of the point.
**/
@:forward
abstract BlockPos(Vec3<Int>) from Vec3<Int> to Vec3<Int> {
	public inline function new(x:Int, y:Int, z:Int) {
		this = new Vec3(x, y, z);
	}

	@:op(A + B)
	public inline function add(rhs:BlockPos):BlockPos {
		return this.add(rhs);
	}

	@:op(A - B)
	public inline function sub(rhs:BlockPos):BlockPos {
		return this.sub(rhs);
	}

	@:op(A * B)
	public inline function multiplyScalar(rhs:Int):BlockPos {
		return this.multiplyScalar(rhs);
	}

	@:op(-A)
	public inline function negate():BlockPos {
		return this.negate();
	}

	@:op(A == B)
	public function equals(rhs:BlockPos):Bool {
		return this.x == rhs.x && this.y == rhs.y && this.z == rhs.z;
	}

	@:op(A != B)
	public function notEquals(rhs:BlockPos):Bool {
		return !equals(rhs);
	}

	public function hashCode():Int {
		return (this.x * 73856093) ^ (this.y * 19349663) ^ (this.z * 83492791);
	}

	public function toString():String {
		return 'BlockPos(${this.x}, ${this.y}, ${this.z})';
	}

	/**
		Returns the chunk the position is in.
	**/
	public function chunk():Vec2<Int> {
		var x = Math.floor(this.x / 16);
		var z = Math.floor(this.z / 16);

		return new Vec2(x, z);
	}

	/**
		Returns a list of positions neighboring the block. No Diagonal.
	**/
	public function neighbors():Array<BlockPos> {
		return [
			new BlockPos(this.x + 1, this.y + 0, this.z + 0), // Front
			new BlockPos(this.x - 1, this.y + 0, this.z + 0), // Back
			new BlockPos(this.x + 0, this.y + 0, this.z - 1), // Left
			new BlockPos(this.x + 0, this.y + 0, this.z + 1), // Right
			new BlockPos(this.x + 0, this.y - 1, this.z + 0), // Bot
			new BlockPos(this.x + 0, this.y + 1, this.z + 0), // Top
		];
	}
}
